local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

Capitalist.componentPriceDefault = 100

function Capitalist:BuildOptions()
	self:DebugEntry("BuildOptions")

	self.options = {
		type = "group",
		args = {
			general = {
				name = L["General"],
				type = "group",
				order = 1,
				args = {
					minProfit = {
						name = L["Minimal profit"],
						order = 1,
						desc = L["How much you want to gain at minimum from each product (price is simply added to material cost)."],
						type = "input",
						set = function(info, val)
								self.db.profile.minProfit = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.minProfit / 10000
							  end,
					},
					margin = {
						name = L["Price margin"],
						order = 2,
						desc = L["The increase (in percent) in price, if there is no or very expensive competition."],
						type = "input",
						set = function(info, val)
								self.db.profile.margin = tonumber(val) or 0
							  end,
						get = function(info)
								return ""..self.db.profile.margin
							  end,
					},
					minMargin = {
						name = L["Minimal increase"],
						order = 3,
						desc = L["How much (in gold) the price should increase at minimum if there is no or very expensive competition."],
						type = "input",
						set = function(info, val)
								self.db.profile.minMargin = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.minMargin / 10000
							  end,
					},
					maxMargin = {
						name = L["Maximal increase"],
						order = 4,
						desc = L["How much (in gold) the price should increase at maximum if there is no or very expensive competition."],
						type = "input",
						set = function(info, val)
								self.db.profile.maxMargin = (tonumber(val) or 0) * 10000
							  end,
						get = function(info)
								return ""..self.db.profile.maxMargin / 10000
							  end,
					},
					showByDefault = {
						name = L["Show by default"],
						order = 5,
						desc = L["Automatically opens Capitalist when you enter the Auction House"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.showByDefault = val
							  end,
						get = function(info)
								return self.db.profile.showByDefault
							  end,
					},
					Northrend = {
						name = L["Northrend"],
						order = 6,
						desc = L["If checked, enchants using Northrend components will be produced"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Northrend = val
								self:ApplyFilters()
								self:UpdateFilterOptions()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Northrend
							  end,
					},
					BurningCrusade = {
						name = L["Burning Crusade"],
						order = 7,
						desc = L["If checked, enchants using Burning Crusade components will be produced"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.BurningCrusade = val
								self:ApplyFilters()
								self:UpdateFilterOptions()
							  end,
						get = function(info)
								return self.db.profile.filters.global.BurningCrusade
							  end,
					},
					Classic = {
						name = L["Classic"],
						order = 8,
						desc = L["If checked, enchants using Classic components will be produced"],
						type = "toggle",
						set = function(info, val)
								self.db.profile.filters.global.Classic = val
								self:ApplyFilters()
								self:UpdateFilterOptions()
							  end,
						get = function(info)
								return self.db.profile.filters.global.Classic
							  end,
					},
				}
			},
			componentPrices = {
				name = L["Component Prices"],
				type = "group",
				order = 97,
				args = {}
			},
			createdProducts = {
				name = L["Created Products"],
				type = "group",
				order = 98,
				args = {}
			},
			debug = {
				name = L["Debug"],
				type = "group",
				order = 99,
				args = {
					TraceFunctions = {
						name = "Trace function entry/exit",
						order = 1,
						desc = "Output debug text on entry and exit of functions that support it.",
						type = "toggle",
						set = function(info, val)
								self.db.profile.debug.TraceFunctions = val
							  end,
						get = function(info)
								return self.db.profile.debug.TraceFunctions
							  end,
					},
					TraceMisc = {
						name = "Trace misc. state",
						order = 1,
						desc = "Output extra debug information scattered throughout.",
						type = "toggle",
						set = function(info, val)
								self.db.profile.debug.TraceMisc = val
							  end,
						get = function(info)
								return self.db.profile.debug.TraceMisc
							  end,
					},
				}
			},
		}
	}

	-- REVIEW: Add Debug options to clear out professions, components, learned products, etc.
	-- ........BUT keep the counts and filters.

	-- add profession nodes
	local order = 11
	if (self.db.profile[realm] and self.db.profile[realm][character]) then

		for profession, professionData in pairs(self.db.profile[realm][character]) do
			self.options.args[profession] = {
												name = profession,
												type = "group",
												order = order,
												args = {
													filters = {
														name = L["Filters"],
														type = "group",
														order = 1,
														args = {}
													},
													counts = {
														name = L["Counts"],
														type = "group",
														order = 2,
														args = {}
													},
												}
											}

			order = order + 1
		end

		self:UpdateFilterOptions()
		self:UpdateStatistics()
	end	

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Capitalist", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Capitalist")
	self:RegisterChatCommand("Capitalist", function(...) Capitalist:HandleSlashCommand(...) end)
	self:RegisterChatCommand("cap", function(...) Capitalist:HandleSlashCommand(...) end)

	self:DebugExit("BuildOptions")
end

function Capitalist:UpdateFilterOptions()
	self:DebugEntry("UpdateFilterOptions")

	if (self.db.profile[realm] and
		self.db.profile[realm][character]) then

		for profession, professionData in pairs(self.db.profile[realm][character]) do
			if (self.options.args[profession]) then
				self:DebugMisc(profession)

				local t

				-- populate filters
				self.options.args[profession].args.filters.args = {}
				t = self.options.args[profession].args.filters.args

				if (self.db.profile[realm][character][profession]) then
					for skillName, numCreated in pairs(self.db.profile[realm][character][profession]) do
						if (self.db.profile.professions and
							self.db.profile.professions[profession] and
							self.db.profile.professions[profession][skillName]) then

							local product = self.db.profile.professions[profession][skillName]
							self:DebugMisc(skillName)
							self:DebugMisc(product)
	
							if (self:ProductComponentsAreVisible(profession, skillName) and
								self:ProductIsVisible(profession, skillName)) then

								local settingSkillName = skillName
								if (profession == L["Enchanting"]) then
									settingSkillName = L.stripSkillName(skillName)
								end

								t[skillName] = {
									name = settingSkillName,
									desc = product.description,
									type = "toggle",
									set = function(info, val)
											self:ProductSetEnabled(profession, skillName, val)
											self:ApplyFilters()
											self:UpdateFilterOptions()
											end,
									get = function(info)
											return self:ProductIsEnabled(profession, skillName)
											end
								}
							end
						end
					end
	
					-- add filter "all"
					if (self.allFilter == nil) then
						self.allFilter = true
					end
					t["all"] = {
						name = L["all products"],
						desc = L["enable or disable all products"],
						type = "toggle",
						order = 1,
						set = function(info, val)
									Capitalist.allFilter = val
									self:ApplyFilters()
									self:UpdateFilterOptions()
								end,
						get = function(info)
									return Capitalist.allFilter
								end
					}

					-- populate component prices
					self.options.args.componentPrices.args = {}
					t = self.options.args.componentPrices.args
					--for componentName, componentData in pairs(self.db.profile.components) do
					local componentList = self:GetVisibleComponents()
					for i = 1, #componentList do
						local componentName = componentList[i]

						--if (self:ComponentIsVisible(componentName)) then
						if (true) then
							t[componentName] = {
								name = componentName,
								desc = string.format(L["Set the price of %s in gold"], componentName),
								type = "range",
								min = 0,
								max = 40,
								step = 0.01,
								bigStep = 1,
								order = self:GetComponentId(componentName),
								set = function(info, val)
											self.db.profile.components[componentName].price = val
										end,
								get = function(info)
											return self:GetComponentPrice(componentName)
										end
							}
						end
					end
	
					-- populate product count
					self.options.args[profession].args.counts.args = {}
					t = self.options.args[profession].args.counts.args

					for skillName, numCreated in pairs(self.db.profile[realm][character][profession]) do
						if (self.db.profile.professions[profession] and
							self.db.profile.professions[profession][skillName]) then

							local product = self.db.profile.professions[profession][skillName]
							self:DebugMisc(skillName)
							self:DebugMisc(product)

							if (self:ProductIsEnabled(profession, skillName) and
								self:ProductIsVisible(profession, skillName)) then

								local settingSkillName = skillName
								if (profession == L["Enchanting"]) then
									settingSkillName = L.stripSkillName(skillName)
								end

								t[skillName] = {
									name = settingSkillName,
									desc = product.description,
									type = "range",
									min = 0,
									max = 40,
									step = 1,
									set = function(info, val)
											self.db.profile.professions[profession][skillName].count = val
											end,
									get = function(info, val)
											return self.db.profile.professions[profession][skillName].count
											end
								}
							end
						end
					end
				end
			end
		end
	end

	self:UpdateStatistics()

	self:DebugExit("UpdateFilterOptions")
end

function Capitalist:UpdateStatistics()
	self:DebugEntry("UpdateStatistics")

	if (self.db.profile[realm] and
		self.db.profile[realm][character]) then

		for profession, professionData in pairs(self.db.profile[realm][character]) do
			if (self.options.args[profession]) then
				self:DebugMisc(profession)

				local t

				-- populate created products
				self.options.args.createdProducts.args = {}
				t = self.options.args.createdProducts.args

				for skillName, numCreated in pairs(self.db.profile[realm][character][profession]) do
					if (self.db.profile.professions[profession] and
						self.db.profile.professions[profession][skillName]) then

						local product = self.db.profile.professions[profession][skillName]

						if (numCreated > 0) then
							t[skillName] = {
								name = numCreated.."x "..product.link,
								type = "description",
								order = -numCreated,
							}
						end
					end
				end
			end
		end
	end

	self:DebugExit("UpdateStatistics")
end

function Capitalist:ReportTop10(...)
	self:DebugEntry("ReportTop10")

	local tableSorted = {}

	if (self.db.profile[realm] and
		self.db.profile[realm][character]) then

		for profession, professionData in pairs(self.db.profile[realm][character]) do
			for skillName, numCreated in pairs(professionData) do
				if (numCreated > 0) then
					local entry = { c = numCreated, n = skillName }
					table.insert(tableSorted, entry )
				end
			end
		end
	end

	table.sort(tableSorted, function(a, b) return a.c > b.c end)

	for i=1, math.min(#tableSorted, 10) do
		local e = tableSorted[i]
		self:Print(e.c.."x "..e.n)
	end

	self:DebugExit("ReportTop10")
end
